%% SUNDELL AND SAYLOR (2017): AN INVERSE MODEL FOR UNMIXING DETRITAL GEOCHRONOLOGY AGE DISTRIBUTIONS; GEOCHEMISTRY, GEOPHYSICS, GEOSYSTEMS. %%
%% MODEL VERSION 1.1.02 %%

clear all % Clear all existing variables
close all % Close all figures
clc % Clear MATLAB Command Window

[filename, pathname] = uigetfile({'*'},'File Selector');
fullpathname = strcat(pathname, filename);
[numbers1, text1, data_tmp1] = xlsread(fullpathname);
data1 = cell2mat(data_tmp1(2:end,:));

tic

num_sources = 11;
bandwidth = 20;

sink_data = data1(:,1+2*num_sources:end);
source_data = data1(:,1:num_sources*2);

N = num_sources; % Number of source samples

RESULTS_OUT_D{num_sources*2+5,length(sink_data(1,:))/2+1} = [];
for i = 1:length(sink_data(1,:))/2
	RESULTS_OUT_D(1,i+1) = text1(1,num_sources*2+i*2-1);
end
for i = 1:num_sources
	RESULTS_OUT_D(i+1,1) = text1(1,i*2-1);
end

for i = 1:num_sources
	RESULTS_OUT_D(i+num_sources+2,1) = text1(1,i*2-1);
end
RESULTS_OUT_D(num_sources*2+4,1) = {'D mean'};
RESULTS_OUT_D(num_sources*2+5,1) = {'D stdev'};

RESULTS_OUT_V{num_sources*2+5,length(sink_data(1,:))/2+1} = [];
for i = 1:length(sink_data(1,:))/2
	RESULTS_OUT_V(1,i+1) = text1(1,num_sources*2+i*2-1);
end
for i = 1:num_sources
	RESULTS_OUT_V(i+1,1) = text1(1,i*2-1);
end

for i = 1:num_sources
	RESULTS_OUT_V(i+num_sources+2,1) = text1(1,i*2-1);
end
RESULTS_OUT_V(num_sources*2+4,1) = {'V mean'};
RESULTS_OUT_V(num_sources*2+5,1) = {'V stdev'};

RESULTS_OUT_R2{num_sources*2+5,length(sink_data(1,:))/2+1} = [];
for i = 1:length(sink_data(1,:))/2
	RESULTS_OUT_R2(1,i+1) = text1(1,num_sources*2+i*2-1);
end
for i = 1:num_sources
	RESULTS_OUT_R2(i+1,1) = text1(1,i*2-1);
end

for i = 1:num_sources
	RESULTS_OUT_R2(i+num_sources+2,1) = text1(1,i*2-1);
end
RESULTS_OUT_R2(num_sources*2+4,1) = {'R2 mean'};
RESULTS_OUT_R2(num_sources*2+5,1) = {'R2 stdev'};

for TYSON = 1:length(sink_data(1,:))/2
	
	trials = 10000; % Number of randomly generated sets of weights used to scale source distributions for comparison to mixed distribution
	threshold_D = 1; % Set KS test D threshold, only change to conserve memory (i.e., if running > 100,000 trials and you know there are fits better than 0.2, set at 0.2 to retain nothing greater)
	threshold_V = 1; % Basically the same as D threshold, but typically set a little higher since V = max(CDF1-CDF2) + max(CDF2-CDF1)
	threshold_R2 = 0; % Same basically as D and V, except better fits are closer to 1 rather than 0
	PDP_min = 0; % Minimum x (age) for constructing and plotting probability density plots
	PDP_max = 3000; % Maximum x (age) for constructing and plotting probability density plots
	PDP_step = 1; % Interval for constructing and plotting probability density plots, smaller steps make smoother PDPs, but cost compute memory
	plotnum = 10; % number of best fits to retain (and plot) to calculate weight mean and standard dev. contribution from each source sample
	
	data = [sink_data(:,TYSON*2-1:TYSON*2),source_data];
	
	% Make probability density plots
	x = PDP_min:PDP_step:PDP_max;
	for i = 1:N+1;
		m = data(:,i*2-1);
		m = m(isfinite(m(:,1)),:);
		s = bandwidth*ones(length(m),1);
		f = zeros(length(m),length(x));
		for j = 1:length(m);
			f(j,:) = (1./ (s(j)*sqrt(2*pi)) .* exp (  (-((x-m(j)).^2)) ./ (2*((s(j)).^2))  ).*PDP_step);
		end
		pdps(:,i) = ((sum(f, 1))/length(m)).';
	end
	pdp_sink = pdps(:,1); % Mixed sample PDP
	pdp_source = pdps(:,2:end); % All source sample PDPs
	
	% Make cumulative distribution functions
	for i = 1:N+1
		ages(:,i) = (data(:,i*2-1));
	end
	x_all = sort(nonzeros(reshape(ages,[numel(ages),1])));
	x_all(isnan(x_all(:,1)),:) = [];
	x_all(x_all>4500) = []; % Remove ages older than Earth
	x_all(x_all<0) = []; % Remove future ages
	binEdges    =  [-inf ; x_all ; inf];
	for i=1:N+1
		x1 = ages(:,i);
		x1(isnan(x1(:,1)),:) = [];
		binCounts(:,i)  =  histc(nonzeros(x1), binEdges, 1);
		clear x1
	end
	for i=1:N+1
		sumCounts(:,i)  =  cumsum(binCounts(:,i))./sum(binCounts(:,i));
	end
	clear binCounts
	for i=1:N+1
		CDF(:,i)  =  sumCounts(1:end-1, i);
	end
	clear sumCounts
	cdf_sink = CDF(2:end,1); % Mixed sample CDF
	cdf_source = CDF(2:end,2:N+1); % All source sample CDFs
	clear CDF
	
	% Set zero variables
	fit_D = 0; % Keep model results if <= threshold_D
	fit_V = 0; % Keep model results if <= threshold_V
	fit_R2 = 0; % Keep model results if >= threshold_R2
	count = 0; % Keep track of trial number during Monte Carlo model
	
	% Set empty variables; if no model fits exist from Monte Carlo model due to threshold settings then these will remain empty ([])
	Passed_D = [];
	Passed_V = [];
	Passed_R2 = [];
	Passed_cdf_D = [];
	Passed_cdf_V = [];
	Passed_pdp_R2 = [];
	min_D = [];
	min_V = [];
	max_R2 = [];
	Results_D = [];
	Results_V = [];
	Results_R2 = [];
	
	%% MONTE CARLO MODEL: RANDOMLY GENERATE SETS OF WEIGHTS TO SCALE SOURCE DISTRIBUTIONS FOR COMPARISON TO MIXED SAMPLE DISTRIBUTION %%
	
	while count < trials % Run Monte Carlo until reach number of user specified trials
		
		count = count + 1; % Current trial during Monte Carlo model
		
		% Determine randomly generated set of weights for each trial
		x3 = 1:1:N-1;
		samples = x3(randperm(N-1));
		x3 = 1:1:N;
		m = 1/N;
		sampled_y = 0;
		sampled_y(1,N+1) = 1;
		y_line = samples.*m;
		for i = 1:N-1
			tmp_min = max(sampled_y(1,1:samples(1,i)));
			tmp_max = min(nonzeros(sampled_y(1,samples(1,i)+1:end)));
			if tmp_min < y_line(1,i)
				sampled_y(1,samples(1,i)+1) = y_line(1,i) + rand*(tmp_max-y_line(1,i));
			else
				sampled_y(1,samples(1,i)+1) = tmp_min + rand*(tmp_max-tmp_min);
			end
		end
		weights_tmp = diff(sampled_y);
		weights = weights_tmp(randperm(length(weights_tmp))); % Randomly generated set of weights for each trial
		
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		% IF USER-SPECIFIED Licht_approach = 0 THEN RANDOMLY SAMPLED WEIGHTS WILL SCALE ENTIRE SOURCE DISTRIBUTIONS FOR COMPARISON TO MIXED SAMPLE DISTRIBUTION %
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		%if Licht_approach == 0
		
		% Weight and sum all source distributions
		cdf = sum(cdf_source.*(repmat(weights,length(x_all),1)),2);
		pdp = sum(pdp_source.*(repmat(weights,length(x),1)),2);
		
		% Compare randomly weighted sources with mixed sample
		D = max([max(cdf_sink - cdf)],[max(cdf - cdf_sink)]);
		V = max(cdf_sink - cdf) + max(cdf - cdf_sink);
		R2 = ((sum((pdp_sink - mean(pdp_sink)).*(pdp - mean(pdp))))/(sqrt((sum((pdp_sink - mean(pdp_sink)).*(pdp_sink - mean(pdp_sink))))*(sum((pdp - mean(pdp)).*(pdp - mean(pdp)))))))*...
			((sum((pdp_sink - mean(pdp_sink)).*(pdp - mean(pdp))))/(sqrt((sum((pdp_sink - mean(pdp_sink)).*(pdp_sink - mean(pdp_sink))))*(sum((pdp - mean(pdp)).*(pdp - mean(pdp)))))));
		
		D_std = 0;
		V_std = 0;
		R2_std = 0;
		
		%end % End if Licht_approach = 0
		
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		% FILTER AND SORT MODEL RESULTS BASED ON USER SPECIFIED THRESHOLDS, DETERMINE BEST FIT SOURCE CONTRIBUTIONS, PLOT RESULTS %
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		% Keep results if they pass user specified thresholds
		if D <= threshold_D
			fit_D = fit_D + 1;
			Passed_cdf_D(:,fit_D) = cdf;
			Passed_weights_D(fit_D, :) = weights;
			Passed_D(fit_D, 1) = D;
			Passed_x_all_D(:,fit_D) = x_all;
		end
		
		if V <= threshold_V
			fit_V = fit_V + 1;
			Passed_cdf_V(:,fit_V) = cdf;
			Passed_weights_V(fit_V, :) = weights;
			Passed_V(fit_V, 1) = V;
			Passed_x_all_V(:,fit_V) = x_all;
		end
		
		if R2 >= threshold_R2
			fit_R2 = fit_R2 + 1;
			Passed_pdp_R2(:,fit_R2) = pdp;
			Passed_weights_R2(fit_R2, :) = weights;
			Passed_R2(fit_R2, 1) = R2;
		end
		
	end % End once reached number of user specified trials
	
	% Sort and trim results to retain 'plotnum' number of best model fits
	if length(Passed_D) >= plotnum
		Results_D = sortrows([Passed_D, zeros(trials,1), Passed_weights_D, transpose(Passed_cdf_D), transpose(Passed_x_all_D)], 1);
		Results_D = Results_D(1:plotnum,:);
		Results_D_mean_std = transpose([mean(Results_D(1:plotnum,3:N+2));std(Results_D(1:plotnum,3:N+2))]);
	end
	
	if length(Passed_V) >= plotnum
		Results_V = sortrows([Passed_V, zeros(trials,1), Passed_weights_V, transpose(Passed_cdf_V), transpose(Passed_x_all_V)], 1);
		Results_V = Results_V(1:plotnum,:);
		Results_V_mean_std = transpose([mean(Results_V(1:plotnum,3:N+2));std(Results_V(1:plotnum,3:N+2))]);
	end
	
	if length(Passed_R2) >= plotnum
		Results_R2 = flipud(sortrows([Passed_R2, zeros(trials,1), Passed_weights_R2, transpose(Passed_pdp_R2)], 1));
		Results_R2 = Results_R2(1:plotnum,:);
		Results_R2_mean_std = transpose([mean(Results_R2(1:plotnum,3:N+2));std(Results_R2(1:plotnum,3:N+2))]);
	end
	
	% Concatenate mean and standard dev. of best fit source contributions for all metrics
	if length(Passed_D) >= plotnum && length(Passed_V) >= plotnum && length(Passed_R2) >= plotnum
		Results_All_mean_std = [Results_D_mean_std, Results_V_mean_std, Results_R2_mean_std];
	end
	
	D_mean = mean(Results_D(:,1));
	V_mean = mean(Results_V(:,1));
	R2_mean = mean(Results_R2(:,1));
	
	D_std = std(Results_D(:,1));
	V_std = std(Results_V(:,1));
	R2_std = std(Results_R2(:,1));
	
	RESULTS_OUT_D(2:num_sources+1,TYSON+1) = num2cell(Results_D_mean_std(:,1));
	RESULTS_OUT_D(num_sources+3:num_sources*2+2,TYSON+1) = num2cell(Results_D_mean_std(:,2));
	RESULTS_OUT_D(num_sources*2+4,TYSON+1) = num2cell(D_mean);
	RESULTS_OUT_D(num_sources*2+5,TYSON+1) = num2cell(D_std);
	
	RESULTS_OUT_V(2:num_sources+1,TYSON+1) = num2cell(Results_V_mean_std(:,1));
	RESULTS_OUT_V(num_sources+3:num_sources*2+2,TYSON+1) = num2cell(Results_V_mean_std(:,2));
	RESULTS_OUT_V(num_sources*2+4,TYSON+1) = num2cell(V_mean);
	RESULTS_OUT_V(num_sources*2+5,TYSON+1) = num2cell(V_std);

	RESULTS_OUT_R2(2:num_sources+1,TYSON+1) = num2cell(Results_R2_mean_std(:,1));
	RESULTS_OUT_R2(num_sources+3:num_sources*2+2,TYSON+1) = num2cell(Results_R2_mean_std(:,2));
	RESULTS_OUT_R2(num_sources*2+4,TYSON+1) = num2cell(R2_mean);
	RESULTS_OUT_R2(num_sources*2+5,TYSON+1) = num2cell(R2_std);
	
	clearvars -except numbers1 text1 data_tmp1 data1 num_sources bandwidth sink_data source_data N TYSON D_mean V_mean R2_mean D_std V_std R2_std RESULTS_OUT_D RESULTS_OUT_V RESULTS_OUT_R2
	
	TYSON
	
end

toc

